@@ -356,7 +356,7 @@ static size_t write_iso_5428_1984(yaz_iconv_t cd, yaz_iconv_encoder_t en,
             yaz_iconv_set_errno(cd, YAZ_ICONV_EILSEQ);
             return (size_t) -1;
         }
-        out[k++] = x;
+        out[k++] = (unsigned char ) x;
         break;
     }
     *outbytesleft -= k;
