@@ -692,6 +692,7 @@ struct rdf_lookup_info {
     NMEM nmem;
     struct rdf_lookup_info *next;
     int debug;
+    int timeout;
     char *xpath;
     char *server;
     char *method;
@@ -700,7 +701,9 @@ struct rdf_lookup_info {
 };
 
 static struct rdf_lookup_info *construct_one_rdf_lookup(NMEM nmem,
-                                                        const xmlNode *ptr, WRBUF wr_error)
+                                                        const xmlNode *ptr,
+                                                        WRBUF wr_error,
+                                                        int timeout)
 {
     struct _xmlAttr *attr;
     struct rdf_lookup_info *info = nmem_malloc(nmem, sizeof(*info));
@@ -711,6 +714,7 @@ static struct rdf_lookup_info *construct_one_rdf_lookup(NMEM nmem,
     info->server = 0;
     info->method = 0;
     info->debug = 0;
+    info->timeout = timeout;
     info->namespacelist = 0;
     for (attr = ptr->properties; attr; attr = attr->next)
     {
@@ -727,9 +731,9 @@ static struct rdf_lookup_info *construct_one_rdf_lookup(NMEM nmem,
     ptr = ptr->children;
     for ( ; ptr ; ptr = ptr->next)
     {
-        if ( ptr->type == XML_ELEMENT_NODE )
+        if (ptr->type == XML_ELEMENT_NODE)
         {
-            if (!xmlStrcmp(ptr->name, BAD_CAST "key") )
+            if (!xmlStrcmp(ptr->name, BAD_CAST "key"))
             {
                 for (attr = ptr->properties; attr; attr = attr->next)
                 {
@@ -753,7 +757,7 @@ static struct rdf_lookup_info *construct_one_rdf_lookup(NMEM nmem,
                     }
                 }
             }
-            else if (!xmlStrcmp(ptr->name, BAD_CAST "server") )
+            else if (!xmlStrcmp(ptr->name, BAD_CAST "server"))
             {
                 for (attr = ptr->properties; attr; attr = attr->next)
                 {
@@ -767,7 +771,8 @@ static struct rdf_lookup_info *construct_one_rdf_lookup(NMEM nmem,
                     {
                         info->method = nmem_strdup(nmem, (const char *) attr->children->content);
                     }
-                    else {
+                    else
+                    {
                         wrbuf_printf(wr_error, "Bad attribute '%s'. "
                                      "Expected url or method.", attr->name);
                         return 0;
@@ -795,6 +800,7 @@ static void *construct_rdf_lookup(const xmlNode *ptr,
     char ** namespaces = 0;
     int debug = 0;
     int nns = 0;
+    int timeout = 0;
     struct _xmlAttr *attr;
     if (strcmp((const char *) ptr->name, "rdf-lookup"))
         return 0;
@@ -807,6 +813,11 @@ static void *construct_rdf_lookup(const xmlNode *ptr,
         {
             debug = atoi((const char *) attr->children->content);
         }
+        else if (!xmlStrcmp(attr->name, BAD_CAST "timeout") &&
+            attr->children && attr->children->type == XML_TEXT_NODE)
+        {
+            timeout = atoi((const char *) attr->children->content);
+        }
         else
         {
             wrbuf_printf(wr_error, "Bad attribute '%s' for <rdf-lookup>. "
@@ -816,17 +827,17 @@ static void *construct_rdf_lookup(const xmlNode *ptr,
     }
     nmem = nmem_create();
     namespaces = nmem_malloc(nmem, RDF_LOOKUP_MAX_NAMESPACES *
-                             2 * sizeof(char *) );
+                             2 * sizeof(char *));
     namespaces[0] = 0;
 
     ptr = ptr->children;
     for ( ; ptr ; ptr = ptr->next) {
-        if (ptr->type == XML_ELEMENT_NODE )
+        if (ptr->type == XML_ELEMENT_NODE)
         {
-            if ( !strcmp((const char *)ptr->name, "lookup") )
+            if (!strcmp((const char *)ptr->name, "lookup"))
             {
-                struct rdf_lookup_info *i = construct_one_rdf_lookup(nmem, ptr, wr_error);
-                if ( !i )
+                struct rdf_lookup_info *i = construct_one_rdf_lookup(nmem, ptr, wr_error, timeout);
+                if (!i)
                 {
                     nmem_destroy(nmem);
                     return 0; /* error already in wr_error */
@@ -837,17 +848,17 @@ static void *construct_rdf_lookup(const xmlNode *ptr,
                     i->debug = debug;
                     *next = i;
                     next = &((*next)->next);
-                    if ( ! i->server )
+                    if (! i->server)
                         i->server = nmem_strdup(nmem, defserver);
                     else
                         defserver = i->server;
-                    if ( ! i->method )
+                    if (! i->method)
                         i->method = nmem_strdup(nmem, "GET");
                     yaz_log(YLOG_DEBUG, "lookup: x=%s k[0]:%s, %s %s",
                             i->xpath, i->keys[0], i->method, i->server);
                 }
             }
-            else if ( !strcmp((const char *)ptr->name, "namespace") )
+            else if (!strcmp((const char *)ptr->name, "namespace"))
             {
                 char * prefix = 0;
                 char * href = 0;
@@ -872,7 +883,7 @@ static void *construct_rdf_lookup(const xmlNode *ptr,
                         return 0;
                     }
                 }
-                if ( prefix && href )
+                if (prefix && href)
                 {
                     namespaces[nns++] = prefix;
                     namespaces[nns++] = href;
@@ -888,7 +899,7 @@ static void *construct_rdf_lookup(const xmlNode *ptr,
             else
             {
                 wrbuf_printf(wr_error, "Expected a <lookup> tag under rdf-lookup, not <%s>",
-                             ptr->name );
+                             ptr->name);
                 nmem_destroy(nmem);
                 return 0;
             }
@@ -914,8 +925,9 @@ static void rdf_lookup_debug_comment(xmlNode *n,
                                      int yloglevel)
 {
     WRBUF com = wrbuf_alloc();
-    wrbuf_printf(com, " rdf-lookup %s %s took %g sec",
-                 info->method, wrbuf_cstr(uri), yaz_timing_get_real(tim));
+    wrbuf_printf(com, " rdf-lookup %s ", info->method);
+    wrbuf_puts_replace_str(com, wrbuf_cstr(uri), "--", "%2D%2D");
+    wrbuf_printf(com, " took %g sec", yaz_timing_get_real(tim));
     if (resp)
         wrbuf_printf(com, " and resulted in %d", resp->code);
     if (msg)
@@ -933,21 +945,24 @@ static void rdf_lookup_debug_comment(xmlNode *n,
     wrbuf_destroy(com);
 }
 
-static void rdf_lookup_node(xmlNode *n, xmlXPathContextPtr xpathCtx, struct rdf_lookup_info *info )
+static void rdf_lookup_node(xmlNode *n, xmlXPathContextPtr xpathCtx,
+                            struct rdf_lookup_info *info)
 {
     int i;
     int nkey;
     int done = 0;
     WRBUF uri = wrbuf_alloc();
     xpathCtx->node = n;
-    for ( nkey = 0; !done && info->keys[nkey]; nkey++ )
+    for (nkey = 0; !done && info->keys[nkey]; nkey++)
     {
-        xmlXPathObjectPtr xpo = xmlXPathEvalExpression((const xmlChar *)info->keys[nkey], xpathCtx);
+        xmlXPathObjectPtr xpo =
+            xmlXPathEvalExpression((const xmlChar *)info->keys[nkey], xpathCtx);
         xmlNodeSetPtr fldNodes = xpo->nodesetval;
         yaz_log(YLOG_DEBUG, "lookup_node: %d: %s", nkey, info->keys[nkey]);
-        if ( fldNodes )
+        if (fldNodes)
         {
-            for (i = 0; !done && i < fldNodes->nodeNr; i++) {
+            for (i = 0; !done && i < fldNodes->nodeNr; i++)
+            {
                 xmlNode *f = fldNodes->nodeTab[i];
                 if (f->type == XML_ELEMENT_NODE)
                     f = f->children;
@@ -959,7 +974,8 @@ static void rdf_lookup_node(xmlNode *n, xmlXPathContextPtr xpathCtx, struct rdf_
                         char *keybuf = xmalloc(3*strlen((const char*) f->content)+1);
                         yaz_url_t url = yaz_url_create();
                         yaz_url_set_max_redirects(url, 0); /* we just want the first redirect */
-                        /* yaz_url_set_verbose(url, 1); */ /* REMOVE THIS LATER */
+                        if (info->timeout)
+                            yaz_url_set_timeout(url, info->timeout, 0);
                         yaz_log(YLOG_DEBUG, "Found key '%s'", (const char*) f->content);
                         yaz_encode_uri_component(keybuf, (const char*) f->content);
                         wrbuf_rewind(uri);
@@ -974,13 +990,14 @@ static void rdf_lookup_node(xmlNode *n, xmlXPathContextPtr xpathCtx, struct rdf_
                         if (resp)
                         {
                             yaz_log(YLOG_DEBUG, "resp code %d, headers %p", resp->code, resp->headers);
-                            if ( (resp->code == 302 || resp->code == 200) && resp->headers )
+                            if ((resp->code == 302 || resp->code == 200)
+                                && resp->headers)
                             {
                                 const char *newuri = z_HTTP_header_lookup(resp->headers, "X-Uri");
-                                if ( newuri && *newuri )
+                                if (newuri && *newuri)
                                 {
                                     xmlSetProp(n, (const xmlChar *)"rdf:about",
-                                               (const xmlChar *)newuri );
+                                               (const xmlChar *)newuri);
                                     done = 1;
                                     rdf_lookup_debug_comment(f->parent, uri,
                                                              resp, info, tim, newuri, YLOG_DEBUG);
@@ -990,19 +1007,19 @@ static void rdf_lookup_node(xmlNode *n, xmlXPathContextPtr xpathCtx, struct rdf_
                                     yaz_log(YLOG_LOG, "rdf-lookup: Got no X-Uri for %s",
                                             wrbuf_cstr(uri));
                                     rdf_lookup_debug_comment(f->parent, uri, resp, info, tim,
-                                                             "No X-URI Header in response!", YLOG_LOG );
+                                                             "No X-URI Header in response!", YLOG_LOG);
                                 }
                             }
                             else
                             {
                                 rdf_lookup_debug_comment(f->parent, uri, resp,
-                                                         info, tim, NULL, YLOG_LOG );
+                                                         info, tim, NULL, YLOG_LOG);
                             }
                             if (!done)
                             { /* something went wrong, dump headers and message */
                                 const char *err = yaz_url_get_error(url);
                                 Z_HTTP_Header *r = resp->headers;
-                                for( ; r; r = r->next)
+                                for ( ; r; r = r->next)
                                     yaz_log(YLOG_DEBUG, "  %s: %s", r->name, r->value);
                                 if (resp->content_len > 0)
                                 {
@@ -1017,14 +1034,14 @@ static void rdf_lookup_node(xmlNode *n, xmlXPathContextPtr xpathCtx, struct rdf_
                                                 resp->content_len - i,
                                                 resp->content_buf + i);
                                 }
-                                if ( err && *err )
+                                if (err && *err)
                                     yaz_log(YLOG_LOG, "Error: %s", err);
                             }
                         }
                         else
                         {
                             rdf_lookup_debug_comment(f->parent, uri, resp, info, tim,
-                                                     "NO RESPONSE", YLOG_LOG );
+                                                     "NO RESPONSE", YLOG_LOG);
                         }
                         yaz_timing_destroy(&tim);
                         yaz_url_destroy(url);
@@ -1054,15 +1071,15 @@ static int convert_rdf_lookup(void *rinfo, WRBUF record, WRBUF wr_error)
         xmlChar *out_buf = 0;
         int out_len;
         xmlXPathContextPtr xpathCtx = xmlXPathNewContext(doc);
-        char **ns = info->namespacelist;
-        while ( *ns )
-        {
-            xmlXPathRegisterNs(xpathCtx, (const xmlChar *)*ns,
-                               (const xmlChar *)*(ns+1));
-            ns += 2;
-        }
         if (xpathCtx)
         {
+            char **ns = info->namespacelist;
+            while (*ns)
+            {
+                xmlXPathRegisterNs(xpathCtx, (const xmlChar *)ns[0],
+                                   (const xmlChar *)ns[1]);
+                ns += 2;
+            }
             while (info)
             {
                 xmlXPathObjectPtr xpathObj =
@@ -1079,12 +1096,19 @@ static int convert_rdf_lookup(void *rinfo, WRBUF record, WRBUF wr_error)
                         {
                             xmlNode *ptr = nodes->nodeTab[i];
                             yaz_log(YLOG_DEBUG, " node %d: t=%d n='%s' c='%s'", i, ptr->type,
-                                    (const char*) ptr->name, ptr->content );
+                                    (const char*) ptr->name, ptr->content);
                             rdf_lookup_node(ptr, xpathCtx, info);
                         }
                     }
                     xmlXPathFreeObject(xpathObj);
                 }
+                else
+                {
+                    wrbuf_printf(wr_error,
+                                 "Cannot compile X-Path expr: %s",
+                                 info->xpath);
+                    ret = -1;
+                }
                 info = info->next;
             }
             xmlXPathFreeContext(xpathCtx);
