@@ -28,6 +28,7 @@
 #include <unistd.h>
 #endif
 #include <yaz/thread_create.h>
+#include <yaz/log.h>
 
 #ifdef WIN32
 /* VS 2003 or later has getaddrinfo; older versions do not */
@@ -72,6 +73,15 @@
 #define RESOLVER_THREAD 1
 #endif
 
+#if HAVE_GNUTLS_H
+#if GNUTLS_VERSION_NUMBER >= 0x030109
+#define SET_GNUTLS_SOCKET(ses, socket) gnutls_transport_set_int(ses, socket)
+#else
+#define SET_GNUTLS_SOCKET(ses, socket) \
+    gnutls_transport_set_ptr(ses, (gnutls_transport_ptr_t) (size_t)  socket)
+#endif
+#endif
+
 static void tcpip_close(COMSTACK h);
 static int tcpip_put(COMSTACK h, char *buf, int size);
 static int tcpip_get(COMSTACK h, char **buf, int *bufsize);
@@ -93,12 +103,6 @@ static COMSTACK tcpip_accept(COMSTACK h);
 static const char *tcpip_addrstr(COMSTACK h);
 static void *tcpip_straddr(COMSTACK h, const char *str);
 
-#if 0
-#define TRC(x) x
-#else
-#define TRC(X)
-#endif
-
 #ifndef YAZ_SOCKLEN_T
 #define YAZ_SOCKLEN_T int
 #endif
@@ -120,11 +124,11 @@ typedef struct tcpip_state
     int written;  /* -1 if we aren't writing */
     int towrite;  /* to verify against user input */
     int (*complete)(const char *buf, int len); /* length/complete. */
+    char *bind_host;
 #if HAVE_GETADDRINFO
     struct addrinfo *ai;
     struct addrinfo *ai_connect;
     int ipv6_only;
-    char *bind_host;
 #if RESOLVER_THREAD
     int pipefd[2];
     char *hoststr;
@@ -139,6 +143,7 @@ typedef struct tcpip_state
     struct tcpip_cred_ptr *cred_ptr;
     gnutls_session_t session;
     char cert_fname[256];
+    int use_bye;
 #endif
     char *connect_request_buf;
     int connect_request_len;
@@ -146,8 +151,11 @@ typedef struct tcpip_state
     int connect_response_len;
 } tcpip_state;
 
+static int log_level = 0;
+
 static int tcpip_init(void)
 {
+    static int log_level_set = 0;
 #ifdef WIN32
     static int initialized = 0;
 #endif
@@ -164,6 +172,11 @@ static int tcpip_init(void)
         initialized = 1;
     }
 #endif
+    if (!log_level_set)
+    {
+        log_level = yaz_log_module_level("comstack");
+        log_level_set = 1;
+    }
     return 1;
 }
 
@@ -175,11 +188,10 @@ static struct tcpip_state *tcpip_state_create(void)
     sp->altsize = sp->altlen = 0;
     sp->towrite = sp->written = -1;
     sp->complete = cs_complete_auto;
-
+    sp->bind_host = 0;
 #if HAVE_GETADDRINFO
     sp->ai = 0;
     sp->ai_connect = 0;
-    sp->bind_host = 0;
 #if RESOLVER_THREAD
     sp->hoststr = 0;
     sp->pipefd[0] = sp->pipefd[1] = -1;
@@ -191,6 +203,7 @@ static struct tcpip_state *tcpip_state_create(void)
     sp->cred_ptr = 0;
     sp->session = 0;
     strcpy(sp->cert_fname, "yaz.pem");
+    sp->use_bye = 0;
 #endif
     sp->connect_request_buf = 0;
     sp->connect_request_len = 0;
@@ -239,7 +252,7 @@ COMSTACK tcpip_type(int s, int flags, int protocol, void *vp)
     p->cerrno = 0;
     p->user = 0;
 
-    TRC(fprintf(stderr, "Created new TCPIP comstack h=%p\n", p));
+    yaz_log(log_level, "Created TCP/SSL comstack h=%p", p);
 
     return p;
 }
@@ -334,6 +347,7 @@ COMSTACK ssl_type(int s, int flags, int protocol, void *vp)
 
     sp->session = (gnutls_session_t) vp;
     /* note: we don't handle already opened socket in SSL mode - yet */
+
     return p;
 #else
     return 0;
@@ -355,14 +369,14 @@ COMSTACK yaz_ssl_create(int s, int flags, int protocol,
 #if HAVE_GNUTLS_H
 static int ssl_check_error(COMSTACK h, tcpip_state *sp, int res)
 {
-    TRC(fprintf(stderr, "ssl_check_error error=%d fatal=%d msg=%s\n",
-                res,
-                gnutls_error_is_fatal(res),
-                gnutls_strerror(res)));
+    yaz_log(log_level, "ssl_check_error error=%d fatal=%d msg=%s",
+            res,
+            gnutls_error_is_fatal(res),
+            gnutls_strerror(res));
     if (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED)
     {
         int dir = gnutls_record_get_direction(sp->session);
-        TRC(fprintf(stderr, " -> incomplete dir=%d\n", dir));
+        yaz_log(log_level, " -> incomplete dir=%d", dir);
         h->io_pending = dir ? CS_WANT_WRITE : CS_WANT_READ;
         return 1;
     }
@@ -432,9 +446,9 @@ struct addrinfo *tcpip_getaddrinfo(const char *str, const char *port,
     return res;
 }
 
-#endif
+#else
 /* gethostbyname .. old systems */
-int tcpip_strtoaddr_ex(const char *str, struct sockaddr_in *add,
+static int tcpip_strtoaddr_ex(const char *str, struct sockaddr_in *add,
                        int default_port)
 {
     struct hostent *hp;
@@ -445,7 +459,7 @@ int tcpip_strtoaddr_ex(const char *str, struct sockaddr_in *add,
 #else
     in_addr_t tmpadd;
 #endif
-    TRC(fprintf(stderr, "tcpip_strtoaddress: %s\n", str ? str : "NULL"));
+    yaz_log(log_level, "tcpip_strtoaddr_ex %s", str ? str : "NULL");
     add->sin_family = AF_INET;
     strncpy(buf, str, sizeof(buf)-1);
     buf[sizeof(buf)-1] = 0;
@@ -474,6 +488,7 @@ int tcpip_strtoaddr_ex(const char *str, struct sockaddr_in *add,
         return 0;
     return 1;
 }
+#endif
 
 #if HAVE_GETADDRINFO
 static struct addrinfo *create_net_socket(COMSTACK h)
@@ -504,7 +519,7 @@ static struct addrinfo *create_net_socket(COMSTACK h)
     }
     if (s == -1)
         return 0;
-    TRC(fprintf(stderr, "First socket fd=%d\n", s));
+    yaz_log(log_level, "First socket fd=%d", s);
     assert(ai);
     h->iofile = s;
     if (ai->ai_family == AF_INET6 && sp->ipv6_only >= 0 &&
@@ -686,7 +701,7 @@ static int cont_connect(COMSTACK h)
 #else
             close(h->iofile);
 #endif
-            TRC(fprintf(stderr, "Other socket call fd=%d\n", s));
+            yaz_log(log_level, "Other socket call fd=%d", s);
             h->state = CS_ST_UNBND;
             h->iofile = s;
             tcpip_set_blocking(h, h->flags);
@@ -713,7 +728,7 @@ int tcpip_connect(COMSTACK h, void *address)
     struct sockaddr_in *add = (struct sockaddr_in *) address;
 #endif
     int r;
-    TRC(fprintf(stderr, "tcpip_connect h=%p\n", h));
+    yaz_log(log_level, "tcpip_connect h=%p", h);
     h->io_pending = 0;
     if (h->state != CS_ST_UNBND)
     {
@@ -758,7 +773,7 @@ int tcpip_connect(COMSTACK h, void *address)
 #else
         if (yaz_errno() == EINPROGRESS)
         {
-            TRC(fprintf(stderr, "Pending fd=%d\n", h->iofile));
+            yaz_log(log_level, "Pending fd=%d", h->iofile);
             h->event = CS_CONNECT;
             h->state = CS_ST_CONNECTING;
             h->io_pending = CS_WANT_WRITE|CS_WANT_READ;
@@ -779,7 +794,7 @@ int tcpip_connect(COMSTACK h, void *address)
 int tcpip_rcvconnect(COMSTACK h)
 {
     tcpip_state *sp = (tcpip_state *)h->cprivate;
-    TRC(fprintf(stderr, "tcpip_rcvconnect\n"));
+    yaz_log(log_level, "tcpip_rcvconnect h=%p", h);
 
     if (h->state == CS_ST_DATAXFER)
         return 0;
@@ -809,22 +824,20 @@ int tcpip_rcvconnect(COMSTACK h)
         {
             r = tcpip_put(h, sp->connect_request_buf,
                           sp->connect_request_len);
-            TRC(fprintf(stderr, "tcpip_put CONNECT r=%d\n", r));
+            yaz_log(log_level, "tcpip_rcvconnect connect put r=%d", r);
             h->event = CS_CONNECT; /* because tcpip_put sets it */
             if (r) /* < 0 is error, 1 is in-complete */
                 return r;
-            TRC(fprintf(stderr, "tcpip_put CONNECT complete\n"));
-            TRC(fwrite(sp->connect_request_buf, 1, sp->connect_request_len, stderr));
+            yaz_log(log_level, "tcpip_rcvconnect connect complete");
         }
         sp->connect_request_len = 0;
 
         r = tcpip_get(h, &sp->connect_response_buf, &sp->connect_response_len);
-        TRC(fprintf(stderr, "tcpip_get CONNECT r=%d\n", r));
+        yaz_log(log_level, "tcpip_rcvconnect connect get r=%d", r);
         if (r == 1)
             return r;
         if (r <= 0)
             return -1;
-        TRC(fwrite(sp->connect_response_buf, 1, r, stderr));
         xfree(sp->connect_request_buf);
         sp->connect_request_buf = 0;
         sp->complete = cs_complete_auto;
@@ -834,13 +847,11 @@ int tcpip_rcvconnect(COMSTACK h)
     {
         tcpip_create_cred(h);
         gnutls_init(&sp->session, GNUTLS_CLIENT);
+        sp->use_bye = 1; /* only say goodbye in client */
         gnutls_set_default_priority(sp->session);
         gnutls_credentials_set (sp->session, GNUTLS_CRD_CERTIFICATE,
                                 sp->cred_ptr->xcred);
-        /* cast to intermediate size_t to avoid GCC warning. */
-        gnutls_transport_set_ptr(sp->session,
-                                 (gnutls_transport_ptr_t)
-                                 (size_t) h->iofile);
+        SET_GNUTLS_SOCKET(sp->session, h->iofile);
     }
     if (sp->session)
     {
@@ -858,9 +869,6 @@ int tcpip_rcvconnect(COMSTACK h)
     return 0;
 }
 
-#define CERTF "ztest.pem"
-#define KEYF "ztest.pem"
-
 static int tcpip_bind(COMSTACK h, void *address, int mode)
 {
     int r;
@@ -876,6 +884,7 @@ static int tcpip_bind(COMSTACK h, void *address, int mode)
     int one = 1;
 #endif
 
+    yaz_log(log_level, "tcpip_bind h=%p", h);
 #if HAVE_GETADDRINFO
 #if RESOLVER_THREAD
     if (sp->pipefd[0] != -1)
@@ -897,12 +906,14 @@ static int tcpip_bind(COMSTACK h, void *address, int mode)
                                                    GNUTLS_X509_FMT_PEM);
         if (res != GNUTLS_E_SUCCESS)
         {
+            yaz_log(log_level, "gnutls_certificate_set_x509_key_file r=%d fatal=%d msg=%s",
+                        res,
+                        gnutls_error_is_fatal(res),
+                        gnutls_strerror(res));
             h->cerrno = CSERRORSSL;
             return -1;
         }
     }
-#else
-    TRC(fprintf(stderr, "tcpip_bind\n"));
 #endif
 #ifndef WIN32
     if (setsockopt(h->iofile, SOL_SOCKET, SO_REUSEADDR, (char*)
@@ -946,7 +957,7 @@ int tcpip_listen(COMSTACK h, char *raddr, int *addrlen,
     YAZ_SOCKLEN_T len = sizeof(addr);
 #endif
 
-    TRC(fprintf(stderr, "tcpip_listen pid=%d\n", getpid()));
+    yaz_log(log_level, "tcpip_listen h=%p", h);
     if (h->state != CS_ST_IDLE)
     {
         h->cerrno = CSOUTSTATE;
@@ -974,11 +985,7 @@ int tcpip_listen(COMSTACK h, char *raddr, int *addrlen,
             h->cerrno = CSNODATA;
         else
         {
-#ifdef WIN32
-            shutdown(h->iofile, SD_RECEIVE);
-#else
-            shutdown(h->iofile, SHUT_RD);
-#endif
+            shutdown(h->iofile, 0); /* SHUT_RD/SHUT_RECEIVE */
             listen(h->iofile, SOMAXCONN);
             h->cerrno = CSYSERR;
         }
@@ -1016,7 +1023,7 @@ COMSTACK tcpip_accept(COMSTACK h)
     unsigned long tru = 1;
 #endif
 
-    TRC(fprintf(stderr, "tcpip_accept h=%p pid=%d\n", h, getpid()));
+    yaz_log(log_level, "tcpip_accept h=%p", h);
     if (h->state == CS_ST_INCON)
     {
 #if HAVE_GNUTLS_H
@@ -1080,10 +1087,7 @@ COMSTACK tcpip_accept(COMSTACK h)
                 xfree(state);
                 return 0;
             }
-            /* cast to intermediate size_t to avoid GCC warning. */
-            gnutls_transport_set_ptr(state->session,
-                                     (gnutls_transport_ptr_t)
-                                     (size_t) cnew->iofile);
+            SET_GNUTLS_SOCKET(state->session, cnew->iofile);
         }
 #endif
         h = cnew;
@@ -1099,14 +1103,13 @@ COMSTACK tcpip_accept(COMSTACK h)
             {
                 if (ssl_check_error(h, state, res))
                 {
-                    TRC(fprintf(stderr, "gnutls_handshake int in tcpip_accept\n"));
+                    yaz_log(log_level, "tcpip_accept gnutls_handshake interrupted");
                     return h;
                 }
-                TRC(fprintf(stderr, "gnutls_handshake failed in tcpip_accept\n"));
+                yaz_log(log_level, "tcpip_accept gnutls_handshake failed");
                 cs_close(h);
                 return 0;
             }
-            TRC(fprintf(stderr, "SSL_accept complete. gnutls\n"));
         }
 #endif
     }
@@ -1134,11 +1137,10 @@ int tcpip_get(COMSTACK h, char **buf, int *bufsize)
     int tmpi, berlen, rest, req, tomove;
     int hasread = 0, res;
 
-    TRC(fprintf(stderr, "tcpip_get: h=%p bufsize=%d\n", h, *bufsize));
+    yaz_log(log_level, "tcpip_get h=%p bufsize=%d", h, *bufsize);
     if (sp->altlen) /* switch buffers */
     {
-        TRC(fprintf(stderr, "  %d bytes in altbuf (%p)\n", sp->altlen,
-                    sp->altbuf));
+        yaz_log(log_level, "  %d bytes in altbuf (%p)", sp->altlen, sp->altbuf);
         tmpc = *buf;
         tmpi = *bufsize;
         *buf = sp->altbuf;
@@ -1172,7 +1174,7 @@ int tcpip_get(COMSTACK h, char **buf, int *bufsize)
                                      CS_TCPIP_BUFCHUNK);
             if (res == 0)
             {
-                TRC(fprintf(stderr, "gnutls_record_recv returned 0\n"));
+                yaz_log(log_level, "gnutls_record_recv returned 0");
                 return 0;
             }
             else if (res < 0)
@@ -1191,11 +1193,11 @@ int tcpip_get(COMSTACK h, char **buf, int *bufsize)
                when EWOULDBLOCK etc. would be required (res = -1) */
 #endif
             res = recv(h->iofile, *buf + hasread, CS_TCPIP_BUFCHUNK, 0);
-            TRC(fprintf(stderr, "  recv res=%d, hasread=%d\n", res, hasread));
+            yaz_log(log_level, "  recv res=%d, hasread=%d", res, hasread);
             if (res < 0)
             {
-                TRC(fprintf(stderr, "  recv errno=%d, (%s)\n", yaz_errno(),
-                            strerror(yaz_errno())));
+                yaz_log(log_level, "  recv errno=%d, (%s)", yaz_errno(),
+                            strerror(yaz_errno()));
 #ifdef WIN32
                 if (WSAGetLastError() == WSAEWOULDBLOCK)
                 {
@@ -1242,8 +1244,8 @@ int tcpip_get(COMSTACK h, char **buf, int *bufsize)
             return -1;
         }
     }
-    TRC(fprintf(stderr, "  Out of read loop with hasread=%d, berlen=%d\n",
-                hasread, berlen));
+    yaz_log(log_level, "  Out of read loop with hasread=%d, berlen=%d",
+                hasread, berlen);
     /* move surplus buffer (or everything if we didn't get a BER rec.) */
     if (hasread > berlen)
     {
@@ -1264,8 +1266,8 @@ int tcpip_get(COMSTACK h, char **buf, int *bufsize)
                 h->cerrno = CSYSERR;
                 return -1;
             }
-        TRC(fprintf(stderr, "  Moving %d bytes to altbuf(%p)\n", tomove,
-                    sp->altbuf));
+        yaz_log(log_level, "  Moving %d bytes to altbuf(%p)", tomove,
+                sp->altbuf);
         memcpy(sp->altbuf, *buf + berlen, sp->altlen = tomove);
     }
     if (berlen < CS_TCPIP_BUFCHUNK - 1)
@@ -1284,7 +1286,7 @@ int tcpip_put(COMSTACK h, char *buf, int size)
     int res;
     struct tcpip_state *state = (struct tcpip_state *)h->cprivate;
 
-    TRC(fprintf(stderr, "tcpip_put: h=%p size=%d\n", h, size));
+    yaz_log(log_level, "tcpip_put h=%p size=%d", h, size);
     h->io_pending = 0;
     h->event = CS_DATA;
     if (state->towrite < 0)
@@ -1341,7 +1343,7 @@ int tcpip_put(COMSTACK h, char *buf, int size)
 #endif
                     )
                 {
-                    TRC(fprintf(stderr, "  Flow control stop\n"));
+                    yaz_log(log_level, "  Flow control stop");
                     h->io_pending = CS_WANT_WRITE;
                     return 1;
                 }
@@ -1355,11 +1357,11 @@ int tcpip_put(COMSTACK h, char *buf, int size)
             }
         }
         state->written += res;
-        TRC(fprintf(stderr, "  Wrote %d, written=%d, nbytes=%d\n",
-                    res, state->written, size));
+        yaz_log(log_level, "  Wrote %d, written=%d, nbytes=%d",
+                res, state->written, size);
     }
     state->towrite = state->written = -1;
-    TRC(fprintf(stderr, "  Ok\n"));
+    yaz_log(log_level, "  Ok");
     return 0;
 }
 
@@ -1367,9 +1369,9 @@ void tcpip_close(COMSTACK h)
 {
     tcpip_state *sp = (struct tcpip_state *)h->cprivate;
 
-    TRC(fprintf(stderr, "tcpip_close: h=%p pid=%d\n", h, getpid()));
-#if HAVE_GETADDRINFO
+    yaz_log(log_level, "tcpip_close: h=%p", h);
     xfree(sp->bind_host);
+#if HAVE_GETADDRINFO
 #if RESOLVER_THREAD
     if (sp->pipefd[0] != -1)
     {
@@ -1383,8 +1385,11 @@ void tcpip_close(COMSTACK h)
     if (h->iofile != -1)
     {
 #if HAVE_GNUTLS_H
-        if (sp->session)
+        if (sp->session && sp->use_bye)
+        {
+            yaz_log(log_level, "tcpip_close: gnutls_bye");
             gnutls_bye(sp->session, GNUTLS_SHUT_WR);
+        }
 #endif
 #ifdef WIN32
         closesocket(h->iofile);
@@ -1405,8 +1410,8 @@ void tcpip_close(COMSTACK h)
 
         if (--(sp->cred_ptr->ref) == 0)
         {
-            TRC(fprintf(stderr, "Removed credentials %p pid=%d\n",
-                        sp->cred_ptr->xcred, getpid()));
+            yaz_log(log_level, "tcpip_close: removed credentials h=%p",
+                    sp->cred_ptr->xcred);
             gnutls_certificate_free_credentials(sp->cred_ptr->xcred);
             xfree(sp->cred_ptr);
         }
@@ -1559,22 +1564,22 @@ static void x509_crt_print(gnutls_x509_crt_t cert)
     /* Print the serial number of the certificate. */
     size = sizeof(serial);
     gnutls_x509_crt_get_serial(cert, serial, &size);
-    
+
     printf("\tCertificate serial number: %s\n", bin2hex(serial, size));
-    
+
     /* Extract some of the public key algorithm's parameters
      */
     algo = gnutls_x509_crt_get_pk_algorithm(cert, &bits);
-    
+
     printf("Certificate public key: %s", gnutls_pk_algorithm_get_name(algo));
-    
+
     /* Print the version of the X.509 certificate. */
     printf("\tCertificate version: #%d\n", gnutls_x509_crt_get_version(cert));
-    
+
     size = sizeof(dn);
     gnutls_x509_crt_get_dn(cert, dn, &size);
     printf("\tDN: %s\n", dn);
-    
+
     size = sizeof(dn);
     gnutls_x509_crt_get_issuer_dn(cert, dn, &size);
     printf("\tIssuer's DN: %s\n", dn);
@@ -1605,7 +1610,7 @@ void cs_print_session_info(COMSTACK cs)
 #endif
             gnutls_x509_crt_init(&cert);
             gnutls_x509_crt_import(cert, &cert_list[i], GNUTLS_X509_FMT_DER);
-            printf("Certificate info %d:\n", i + 1);
+            printf("Certificate info %u:\n", i + 1);
 #if USE_GNUTLS_X509_CRT_PRINT
             ret = gnutls_x509_crt_print(cert, GNUTLS_CRT_PRINT_FULL,
                                         &cinfo);
